@@ -24,6 +24,15 @@ gboolean ibus_chewing_engine_process_key_event(IBusEngine * engine,
     IBUS_CHEWING_LOG(MSG, "process_key_event() result=%d", result);
     self_update(self);
 
+    if (kSym == IBUS_KEY_Shift_L || kSym == IBUS_KEY_Shift_R || 
+        kSym == IBUS_KEY_Caps_Lock) {
+        /* Refresh property list (language bar) only when users toggle
+         * Chi-Eng Mode or Shape Mode with Shift or Caps Lock, otherwise
+         * the bar will stick to the cursor and block the candidats list.
+         */
+        self_refresh_property_list(self);
+    }
+
     return result;
 }
 
@@ -83,11 +92,14 @@ void ibus_chewing_engine_property_activate(IBusEngine * engine,
             }
         }
         self_refresh_property(self, prop_name);
-    } else if (STRING_EQUALS(prop_name, "chewing_alnumSize_prop")) {
+    } else if (STRING_EQUALS(prop_name, "AlnumSize")) {
         /* Toggle Full <-> Half */
         chewing_set_ShapeMode(self->icPreEdit->context,
                               !chewing_get_ShapeMode(self->icPreEdit->context));
         self_refresh_property(self, prop_name);
+    } else if (STRING_EQUALS(prop_name, "setup_prop")) {
+        /* open preferences window */
+        system(QUOTE_ME(LIBEXEC_DIR) "/ibus-setup-chewing");
     } else {
         IBUS_CHEWING_LOG(DEBUG,
                          "property_activate(-, %s, %u) not recognized",
