@@ -17,7 +17,9 @@ void ibus_chewing_engine_start(IBusChewingEngine * self)
         IBUS_ENGINE_GET_CLASS(self)->property_show(IBUS_ENGINE(self),
                                                    "InputMode");
         IBUS_ENGINE_GET_CLASS(self)->property_show(IBUS_ENGINE(self),
-                                                   "chewing_alnumSize_prop");
+                                                   "AlnumSize");
+        IBUS_ENGINE_GET_CLASS(self)->property_show(IBUS_ENGINE(self),
+                                                   "setup_prop");
         ibus_engine_register_properties(IBUS_ENGINE(self), self->prop_list);
         ibus_chewing_engine_set_status_flag(self,
                                             ENGINE_FLAG_PROPERTIES_REGISTERED);
@@ -203,20 +205,6 @@ IBusText *decorate_pre_edit(IBusChewingPreEdit * icPreEdit,
                                IBUS_ATTR_UNDERLINE_SINGLE,
                                0, charLen);
 
-    IntervalType it;
-
-    chewing_interval_Enumerate(icPreEdit->context);
-    /* Add double lines on chewing interval that contains cursor */
-    while (chewing_interval_hasNext(icPreEdit->context)) {
-        chewing_interval_Get(icPreEdit->context, &it);
-        if (it.from <= chiSymbolCursor && chiSymbolCursor <= it.to) {
-            ibus_text_append_attribute(iText,
-                                       IBUS_ATTR_TYPE_UNDERLINE,
-                                       IBUS_ATTR_UNDERLINE_DOUBLE,
-                                       it.from, it.to);
-        }
-    }
-
     /* Use background color to show current cursor */
     if (chiSymbolCursor < charLen) {
         ibus_text_append_attribute(iText,
@@ -246,6 +234,7 @@ void update_pre_edit_text(IBusChewingEngine * self)
 {
     refresh_pre_edit_text(self);
     gboolean visible = TRUE;
+    gint bpmfLen = self->icPreEdit->bpmfLen;
 
     IBusPreeditFocusMode mode;
 
@@ -258,7 +247,7 @@ void update_pre_edit_text(IBusChewingEngine * self)
 
     parent_update_pre_edit_text_with_mode(IBUS_ENGINE(self),
                                           self->preEditText,
-                                          cursor_current, visible, mode);
+                                          cursor_current + bpmfLen, visible, mode);
 }
 
 void refresh_aux_text(IBusChewingEngine * self)
@@ -272,6 +261,11 @@ void refresh_aux_text(IBusChewingEngine * self)
     /* Make auxText (text to be displayed in auxiliary candidate window).
      * Use auxText to show messages from libchewing, such as "已有：".
      */
+
+    gboolean showPageNumber =
+        ibus_chewing_pre_edit_get_property_boolean(self->icPreEdit,
+                                                   "show-page-number");
+
     if (chewing_aux_Length(self->icPreEdit->context) > 0) {
         IBUS_CHEWING_LOG(INFO, "update_aux_text() chewing_aux_Length=%x",
                          chewing_aux_Length(self->icPreEdit->context));
@@ -280,6 +274,10 @@ void refresh_aux_text(IBusChewingEngine * self)
         IBUS_CHEWING_LOG(INFO, "update_aux_text() auxStr=%s", auxStr);
         self->auxText = g_object_ref_sink(ibus_text_new_from_string(auxStr));
         g_free(auxStr);
+    } else if (showPageNumber && (chewing_cand_TotalPage(self->icPreEdit->context) > 0)) {
+        int TotalPage = chewing_cand_TotalPage(self->icPreEdit->context);
+        int currentPage = chewing_cand_CurrentPage(self->icPreEdit->context) + 1;
+        self->auxText = g_object_ref_sink(ibus_text_new_from_printf("(%i/%i)", currentPage, TotalPage));
     } else {
         /* clear out auxText, otherwise it will be displayed continually. */
         self->auxText = g_object_ref_sink(ibus_text_new_from_static_string(""));
