@@ -58,8 +58,6 @@ gchar *ibus_chewing_pre_edit_get_bopomofo_string(IBusChewingPreEdit * self)
 {
 #if CHEWING_CHECK_VERSION(0,4,0)
     const gchar *buf = chewing_bopomofo_String_static(self->context);
-
-    self->bpmfLen = (gint) g_utf8_strlen(buf, 0);
     return g_strdup(buf);
 #else
     return chewing_zuin_String(self->context, &(self->bpmfLen));
@@ -99,6 +97,7 @@ void ibus_chewing_pre_edit_update(IBusChewingPreEdit * self)
     /* Make preEdit */
     gchar *bufferStr = chewing_buffer_String(self->context);
     gchar *bpmfStr = ibus_chewing_pre_edit_get_bopomofo_string(self);
+    self->bpmfLen = (gint) g_utf8_strlen(bpmfStr, -1);
 
     g_string_assign(self->preEdit, "");
     gint i;
@@ -108,6 +107,7 @@ void ibus_chewing_pre_edit_update(IBusChewingPreEdit * self)
     IBUS_CHEWING_LOG(INFO,
                      "* ibus_chewing_pre_edit_update(-)  bufferStr=|%s|, bpmfStr=|%s| bpmfLen=%d cursor=%d",
                      bufferStr, bpmfStr, self->bpmfLen, cursor_current);
+
     for (i = 0; i < chewing_buffer_Len(self->context) && cP != NULL; i++) {
         if (i == cursor_current) {
             /* Insert bopomofo string */
@@ -120,7 +120,9 @@ void ibus_chewing_pre_edit_update(IBusChewingPreEdit * self)
     if (chewing_buffer_Len(self->context) <= cursor_current) {
         g_string_append(self->preEdit, bpmfStr);
     }
+
     self->wordLen = i + self->bpmfLen;
+
     g_free(bufferStr);
     g_free(bpmfStr);
 
@@ -250,10 +252,11 @@ KSym self_key_sym_fix(IBusChewingPreEdit * self, KSym kSym,
                       KeyModifiers unmaskedMod)
 {
     gchar caseConversionMode = default_english_case_short;
+    gchar toggleChinese = chi_eng_toggle_key;
+
+
+    if (toggleChinese != 'c') {
 
-    if (!ibus_chewing_pre_edit_get_property_boolean(self,
-                                                    "capslock-toggle-chinese"))
-    {
         caseConversionMode = 'n';
     }
     if (is_chinese) {
@@ -292,6 +295,14 @@ EventResponse self_handle_key_sym_default(IBusChewingPreEdit * self,
                                           KSym kSym, KeyModifiers unmaskedMod)
 {
     filter_modifiers(IBUS_SHIFT_MASK);
+
+    if (!is_full_shape && !is_chinese) {
+	/* Users treat English Sub-mode as IM Disabled,
+	 * So key strokes should be passed to client directly. Github 144.
+	 */
+        ignore_when_buffer_is_empty_and_table_not_showing; 
+    }
+
     handle_log("key_sym_default");
 
     /* Seem like we need to disable easy symbol temporarily 
@@ -343,7 +354,13 @@ EventResponse self_handle_num(IBusChewingPreEdit * self, KSym kSym,
                               KeyModifiers unmaskedMod)
 {
     filter_modifiers(IBUS_SHIFT_MASK | IBUS_CONTROL_MASK);
-    absorb_when_release;
+    if ((maskedMod == 0) && !is_full_shape && !is_chinese) {
+	/* Users treat English Sub-mode as IM Disabled,
+	 * So key strokes should be passed to client directly. Github 144.
+	 */
+        ignore_when_buffer_is_empty_and_table_not_showing; 
+    }  
+    ignore_when_release;
     handle_log("num");
 
     if (is_ctrl_only) {
@@ -359,7 +376,13 @@ EventResponse self_handle_num_keypad(IBusChewingPreEdit * self,
                                      KSym kSym, KeyModifiers unmaskedMod)
 {
     filter_modifiers(IBUS_SHIFT_MASK | IBUS_CONTROL_MASK);
-    absorb_when_release;
+    if ((maskedMod == 0) && (!is_full_shape)) {
+	/* Let libchewing handles Keypad keys only when needed.
+	 * Otherwise it might cause some issues. Github 144.
+	 */
+        ignore_when_buffer_is_empty_and_table_not_showing; 
+    }
+    ignore_when_release;
     handle_log("num_keypad");
 
     KSym kSymEquiv = key_sym_KP_to_normal(kSym);
@@ -392,10 +415,9 @@ EventResponse self_handle_caps_lock(IBusChewingPreEdit * self, KSym kSym,
                                     KeyModifiers unmaskedMod)
 {
     filter_modifiers(0);
+    gchar toggleChinese = chi_eng_toggle_key;
 
-    if (!ibus_chewing_pre_edit_get_property_boolean(self,
-                                                    "capslock-toggle-chinese"))
-    {
+    if (toggleChinese != 'c') {
         /* Ignore the Caps Lock event when it does not toggle Chinese */
         return EVENT_RESPONSE_IGNORE;
     }
@@ -411,17 +433,15 @@ EventResponse self_handle_caps_lock(IBusChewingPreEdit * self, KSym kSym,
     return event_process_or_ignore(!chewing_handle_Capslock(self->context));
 }
 
-EventResponse self_handle_shift(IBusChewingPreEdit * self, KSym kSym,
+EventResponse self_handle_shift_left(IBusChewingPreEdit * self, KSym kSym,
                                 KeyModifiers unmaskedMod)
 {
     filter_modifiers(IBUS_SHIFT_MASK);
-    handle_log("shift");
+    handle_log("shift_left");
 
-    gboolean shiftIsToggleChinese =
-        ibus_chewing_pre_edit_get_property_boolean(self,
-                                                   "shift-toggle-chinese");
+    gchar toggleChinese = chi_eng_toggle_key;
 
-    if (!shiftIsToggleChinese) {
+    if (toggleChinese != 's' && toggleChinese != 'l' ) {
         return EVENT_RESPONSE_IGNORE;
     }
 
@@ -440,11 +460,85 @@ EventResponse self_handle_shift(IBusChewingPreEdit * self, KSym kSym,
     return EVENT_RESPONSE_ABSORB;
 }
 
+EventResponse self_handle_shift_right(IBusChewingPreEdit * self, KSym kSym,
+                                KeyModifiers unmaskedMod)
+{
+    filter_modifiers(IBUS_SHIFT_MASK);
+    handle_log("shift_right");
+
+    gchar toggleChinese = chi_eng_toggle_key;
+
+    if (toggleChinese != 's' && toggleChinese != 'r' ) {
+        return EVENT_RESPONSE_IGNORE;
+    }
+
+    if (!event_is_released(unmaskedMod)) {
+        return EVENT_RESPONSE_ABSORB;
+    }
+
+    /* keyLast != Shift means Shift is just part of combination,
+     * thus should not be recognized as single Shift key
+     */
+    if (self->keyLast != IBUS_KEY_Shift_L && self->keyLast != IBUS_KEY_Shift_R) {
+        return EVENT_RESPONSE_ABSORB;
+    }
+
+    ibus_chewing_pre_edit_toggle_chi_eng_mode(self);
+    return EVENT_RESPONSE_ABSORB;
+}
+
+EventResponse self_handle_page_up(IBusChewingPreEdit * self, KSym kSym,
+                                  KeyModifiers unmaskedMod)
+{
+    filter_modifiers(0);
+    ignore_when_buffer_is_empty_and_table_not_showing;
+    ignore_when_release;
+    handle_log("page_up");
+
+    int currentPage = chewing_cand_CurrentPage(self->context);
+    int hasNext = chewing_cand_list_has_next(self->context);
+    int hasPrev = chewing_cand_list_has_prev(self->context);
+
+    if (table_is_showing && (currentPage == 0)) {
+        if (hasNext == 1 || hasPrev == 1) {
+            return event_process_or_ignore(!chewing_handle_Down(self->context));
+        }
+    }
+
+    return event_process_or_ignore(!chewing_handle_PageUp(self->context));
+}
+
+EventResponse self_handle_page_down(IBusChewingPreEdit * self, KSym kSym,
+                                    KeyModifiers unmaskedMod)
+{
+    filter_modifiers(0);
+    ignore_when_buffer_is_empty_and_table_not_showing;
+    ignore_when_release;
+    handle_log("page_down");
+
+    int totalPage = chewing_cand_TotalPage(self->context);
+    int currentPage = chewing_cand_CurrentPage(self->context);
+
+    if (table_is_showing && (currentPage == totalPage - 1)) {
+        return event_process_or_ignore(!chewing_handle_Down(self->context));
+    }
+
+    return event_process_or_ignore(!chewing_handle_PageDown(self->context));
+}
+
 EventResponse self_handle_space(IBusChewingPreEdit * self, KSym kSym,
                                 KeyModifiers unmaskedMod)
 {
     filter_modifiers(IBUS_SHIFT_MASK | IBUS_CONTROL_MASK);
-    absorb_when_release;
+
+    if (!is_shift_only && !is_chinese && !is_full_shape) { 
+	/* Let libchewing handles Keypad keys only when needed.
+	 * Otherwise it might cause some issues. Github 144.
+	 */
+        ignore_when_buffer_is_empty_and_table_not_showing; 
+    }
+
+    ignore_when_release;
     handle_log("space");
 
     if (is_shift_only) {
@@ -452,6 +546,14 @@ EventResponse self_handle_space(IBusChewingPreEdit * self, KSym kSym,
         return EVENT_RESPONSE_PROCESS;
     }
 
+    /* Bug of libchewing: 
+     * If "Space as selection" is not enabled, Space key cannot be used
+     * to turn pages or switch the length of candidate lis.
+     */
+    if (table_is_showing) {
+        return self_handle_page_down(self, kSym, unmaskedMod);
+    }
+
     /* Bug of libchewing: 
      * when "space to select" is enabled, chewing_handle_Space() will
      * ignore the first space. Therefore, use chewing_handle_Space()
@@ -471,10 +573,16 @@ EventResponse self_handle_return(IBusChewingPreEdit * self, KSym kSym,
                                  KeyModifiers unmaskedMod)
 {
     filter_modifiers(0);
-    ignore_when_buffer_is_empty;
-    absorb_when_release;
+    ignore_when_buffer_is_empty_and_table_not_showing;
+    ignore_when_release;
     handle_log("return");
 
+    if (table_is_showing) {
+        int cursorInPage = ibus_lookup_table_get_cursor_in_page(self->iTable);
+        cursorInPage = chewing_get_selKey(self->context)[cursorInPage];
+        return self_handle_key_sym_default(self, cursorInPage, unmaskedMod);
+    }
+
     EventResponse response =
         event_process_or_ignore(!chewing_handle_Enter(self->context));
 
@@ -490,7 +598,7 @@ EventResponse self_handle_backspace(IBusChewingPreEdit * self, KSym kSym,
 {
     filter_modifiers(0);
     ignore_when_buffer_is_empty_and_table_not_showing;
-    absorb_when_release;
+    absorb_when_release; // Triggers focus-out and focus-in on ignore, so use absorb.
     handle_log("backspace");
 
 #if !CHEWING_CHECK_VERSION(0,4,0)
@@ -506,8 +614,8 @@ EventResponse self_handle_delete(IBusChewingPreEdit * self, KSym kSym,
                                  KeyModifiers unmaskedMod)
 {
     filter_modifiers(0);
-    ignore_when_buffer_is_empty;
-    absorb_when_release;
+    ignore_when_buffer_is_empty_and_table_not_showing;
+    ignore_when_release;
     handle_log("delete");
 
     return event_process_or_ignore(!chewing_handle_Del(self->context));
@@ -518,7 +626,7 @@ EventResponse self_handle_escape(IBusChewingPreEdit * self, KSym kSym,
 {
     filter_modifiers(0);
     ignore_when_buffer_is_empty_and_table_not_showing;
-    absorb_when_release;
+    ignore_when_release;
     handle_log("escape");
 
     return event_process_or_ignore(!chewing_handle_Esc(self->context));
@@ -529,7 +637,7 @@ EventResponse self_handle_left(IBusChewingPreEdit * self, KSym kSym,
 {
     filter_modifiers(IBUS_SHIFT_MASK);
     ignore_when_buffer_is_empty_and_table_not_showing;
-    absorb_when_release;
+    ignore_when_release;
     handle_log("left");
 
     if (is_shift_only) {
@@ -537,6 +645,19 @@ EventResponse self_handle_left(IBusChewingPreEdit * self, KSym kSym,
             event_process_or_ignore(!chewing_handle_ShiftLeft(self->context));
     }
 
+    if (table_is_showing) {
+        if(!ibus_chewing_pre_edit_is_vertical_table(self)) {
+            /* horizontal look-up table */
+            int pos = ibus_lookup_table_get_cursor_in_page(self->iTable);
+            if (pos) {
+                ibus_lookup_table_cursor_up(self->iTable);
+                return EVENT_RESPONSE_ABSORB;
+            }
+        }
+
+        return self_handle_page_up(self, kSym, unmaskedMod);
+    }
+
     return event_process_or_ignore(!chewing_handle_Left(self->context));
 }
 
@@ -545,9 +666,22 @@ EventResponse self_handle_up(IBusChewingPreEdit * self, KSym kSym,
 {
     filter_modifiers(0);
     ignore_when_buffer_is_empty_and_table_not_showing;
-    absorb_when_release;
+    ignore_when_release;
     handle_log("up");
 
+    if (table_is_showing) {
+        if(ibus_chewing_pre_edit_is_vertical_table(self)) {
+            /* vertical look-up table */
+            int pos = ibus_lookup_table_get_cursor_in_page(self->iTable);
+            if (pos) {
+                ibus_lookup_table_cursor_up(self->iTable);
+                return EVENT_RESPONSE_ABSORB;
+            }
+        }
+
+        return self_handle_page_up(self, kSym, unmaskedMod);
+    }
+
     return event_process_or_ignore(!chewing_handle_Up(self->context));
 }
 
@@ -556,7 +690,7 @@ EventResponse self_handle_right(IBusChewingPreEdit * self, KSym kSym,
 {
     filter_modifiers(IBUS_SHIFT_MASK);
     ignore_when_buffer_is_empty_and_table_not_showing;
-    absorb_when_release;
+    ignore_when_release;
     handle_log("right");
 
     if (is_shift_only) {
@@ -564,6 +698,21 @@ EventResponse self_handle_right(IBusChewingPreEdit * self, KSym kSym,
             event_process_or_ignore(!chewing_handle_ShiftRight(self->context));
     }
 
+    if (table_is_showing) {
+
+        if(!ibus_chewing_pre_edit_is_vertical_table(self)) {
+            /* horizontal look-up table */
+            int numberCand = ibus_lookup_table_get_number_of_candidates(self->iTable);
+            int cursorInPage = ibus_lookup_table_get_cursor_in_page(self->iTable) + 1;
+            if (cursorInPage != numberCand) {
+                ibus_lookup_table_cursor_down(self->iTable);
+                return EVENT_RESPONSE_ABSORB;
+            }
+        }
+
+        return self_handle_page_down(self, kSym, unmaskedMod);
+    }
+
     return event_process_or_ignore(!chewing_handle_Right(self->context));
 }
 
@@ -572,52 +721,34 @@ EventResponse self_handle_down(IBusChewingPreEdit * self, KSym kSym,
 {
     filter_modifiers(0);
     ignore_when_buffer_is_empty_and_table_not_showing;
-    absorb_when_release;
+    ignore_when_release;
     handle_log("down");
 
+    if (table_is_showing) {
+        if(ibus_chewing_pre_edit_is_vertical_table(self)) {
+
+            /* vertical look-up table */
+            int numberCand = ibus_lookup_table_get_number_of_candidates(self->iTable);
+            int cursorInPage = ibus_lookup_table_get_cursor_in_page(self->iTable) + 1;
+            if (cursorInPage != numberCand) {
+                ibus_lookup_table_cursor_down(self->iTable);
+                return EVENT_RESPONSE_ABSORB;
+            }
+        }
+
+        /* horizontal look-up table */
+        return self_handle_page_down(self, kSym, unmaskedMod);
+    }
+
     return event_process_or_ignore(!chewing_handle_Down(self->context));
 }
 
-EventResponse self_handle_page_up(IBusChewingPreEdit * self, KSym kSym,
-                                  KeyModifiers unmaskedMod)
-{
-    filter_modifiers(0);
-    ignore_when_buffer_is_empty_and_table_not_showing;
-    absorb_when_release;
-    handle_log("page_up");
-
-#if !CHEWING_CHECK_VERSION(0,4,0)
-    if (table_is_showing) {
-        return event_process_or_ignore(!chewing_handle_Left(self->context));
-    }
-#endif
-
-    return event_process_or_ignore(!chewing_handle_PageUp(self->context));
-}
-
-EventResponse self_handle_page_down(IBusChewingPreEdit * self, KSym kSym,
-                                    KeyModifiers unmaskedMod)
-{
-    filter_modifiers(0);
-    ignore_when_buffer_is_empty_and_table_not_showing;
-    absorb_when_release;
-    handle_log("page_down");
-
-#if !CHEWING_CHECK_VERSION(0,4,0)
-    if (table_is_showing) {
-        return event_process_or_ignore(!chewing_handle_Right(self->context));
-    }
-#endif
-
-    return event_process_or_ignore(!chewing_handle_PageDown(self->context));
-}
-
 EventResponse self_handle_tab(IBusChewingPreEdit * self, KSym kSym,
                               KeyModifiers unmaskedMod)
 {
     filter_modifiers(0);
-    ignore_when_buffer_is_empty;
-    absorb_when_release;
+    ignore_when_buffer_is_empty_and_table_not_showing;
+    ignore_when_release;
     handle_log("tab");
 
     return event_process_or_ignore(!chewing_handle_Tab(self->context));
@@ -627,8 +758,8 @@ EventResponse self_handle_home(IBusChewingPreEdit * self, KSym kSym,
                                KeyModifiers unmaskedMod)
 {
     filter_modifiers(0);
-    ignore_when_buffer_is_empty;
-    absorb_when_release;
+    ignore_when_buffer_is_empty_and_table_not_showing;
+    ignore_when_release;
     handle_log("home");
 
     return event_process_or_ignore(!chewing_handle_Home(self->context));
@@ -638,8 +769,8 @@ EventResponse self_handle_end(IBusChewingPreEdit * self, KSym kSym,
                               KeyModifiers unmaskedMod)
 {
     filter_modifiers(0);
-    ignore_when_buffer_is_empty;
-    absorb_when_release;
+    ignore_when_buffer_is_empty_and_table_not_showing;
+    ignore_when_release;
     handle_log("end");
 
     return event_process_or_ignore(!chewing_handle_End(self->context));
@@ -656,7 +787,7 @@ EventResponse self_handle_default(IBusChewingPreEdit * self, KSym kSym,
                                   KeyModifiers unmaskedMod)
 {
     filter_modifiers(IBUS_SHIFT_MASK);
-    absorb_when_release;
+    ignore_when_release;
     handle_log("default");
 
     return self_handle_key_sym_default(self, kSym, unmaskedMod);
@@ -672,7 +803,10 @@ KeyHandlingRule keyHandlingRules[] = {
     {IBUS_KEY_Caps_Lock, IBUS_KEY_Caps_Lock, self_handle_caps_lock}
     ,
 
-    {IBUS_KEY_Shift_L, IBUS_KEY_Shift_R, self_handle_shift}
+    {IBUS_KEY_Shift_L, IBUS_KEY_Shift_L, self_handle_shift_left}
+    ,
+
+    {IBUS_KEY_Shift_R, IBUS_KEY_Shift_R, self_handle_shift_right}
     ,
 
     {IBUS_KEY_space, IBUS_KEY_space, self_handle_space}
@@ -750,10 +884,12 @@ KeyHandlingRule keyHandlingRules[] = {
     {IBUS_KP_Multiply, IBUS_KP_Divide, self_handle_num_keypad}
     ,
 
-    {128, G_MAXUINT, self_handle_special}
+    {32, 127, self_handle_default}
     ,
-
-    {0, 0, self_handle_default}
+    /* we need only printable ascii characters (32~127) and keys
+     * like enter, esc, backspace ... etc. The rest can be ignored.
+     */
+    {0, G_MAXUINT, self_handle_special}
     ,
 };
 
