@@ -1,4 +1,4 @@
-#define GETTEXT_PACKAGE "gtk20"
+#define GETTEXT_PACKAGE "gtk30"
 #include <glib.h>
 #include <glib/gi18n.h>
 #include "MakerDialogUtil.h"
@@ -66,6 +66,14 @@ const gchar *outputCharsets[] = {
     NULL
 };
 
+const gchar *chiEngToggle[] = {
+    N_("caps_lock"),
+    N_("shift"),
+    N_("shift_l"),
+    N_("shift_r"),
+    NULL
+};
+
 MkdgPropertySpec propSpecs[] = {
     {G_TYPE_STRING, "kb-type", PAGE_KEYBOARD, N_("Keyboard Type"),
      IBUS_CHEWING_PROPERTIES_SUBSECTION, "default", kbType_ids, NULL, 0, 0,
@@ -96,13 +104,13 @@ MkdgPropertySpec propSpecs[] = {
      N_("Auto move cursor"),
      IBUS_CHEWING_PROPERTIES_SUBSECTION, "1", NULL, NULL, 0, 1,
      autoShiftCur_apply_callback, 0,
-     N_("Automatically move cursor to next character"), NULL}
+     N_("Automatically move the cursor to the next character after selection"), NULL}
     ,
     {G_TYPE_BOOLEAN, "add-phrase-direction", PAGE_EDITING,
-     N_("Add phrases to the front"),
+     N_("Add phrase before the cursor"),
      IBUS_CHEWING_PROPERTIES_SUBSECTION, "1", NULL, NULL, 0, 1,
      addPhraseDirection_apply_callback, 0,
-     N_("Add phrases to the front"), NULL}
+     N_("Use Ctrl + Numbers (2-9) to add new phrase before the cursor"), NULL}
     ,
     {G_TYPE_BOOLEAN, "clean-buffer-focus-out", PAGE_EDITING,
      N_("Clean pre-edit buffer when focus out"),
@@ -132,37 +140,27 @@ MkdgPropertySpec propSpecs[] = {
      ("Maximum Chinese characters in pre-edit buffer, not including inputing Zhuyin symbols."),
      NULL}
     ,
-    /* Sync between CapsLock and IM */
-    {G_TYPE_STRING, "sync-caps-lock", PAGE_EDITING,
-     N_("Sync between CapsLock and IM"),
-     IBUS_CHEWING_PROPERTIES_SUBSECTION, "disable", syncCapsLock_strs,
-     "Sync",
-     0, 1,
-     syncCapsLock_apply_callback,
+    {
+     G_TYPE_STRING, 
+     "chi-eng-mode-toggle", 
+     PAGE_EDITING,
+     N_("Chinese/Alphanumeric Mode Toggle Key"),
+     IBUS_CHEWING_PROPERTIES_SUBSECTION, 
+     "caps_lock",
+     chiEngToggle, 
+     NULL, 
+     0, 
+     0,
+     chiEngToggle_apply_callback,
      MKDG_PROPERTY_FLAG_NO_NEW | MKDG_PROPERTY_FLAG_HAS_TRANSLATION,
-     N_
-     ("Occasionally, the CapsLock status does not match the IM, this option determines how these status be synchronized. Valid values:\n"
-      "\"disable\": Do nothing\n"
-      "\"keyboard\": IM status follows keyboard status\n"
-      "\"IM\": Keyboard status follows IM status"), NULL}
-    ,
-    {G_TYPE_BOOLEAN, "shift-toggle-chinese", PAGE_EDITING,
-     N_("Shift toggle Chinese Mode"),
-     IBUS_CHEWING_PROPERTIES_SUBSECTION, "1", NULL, NULL, 0, 1,
-     shiftToggleChinese_apply_callback, 0,
-     N_("Shift key to toggle Chinese Mode"), NULL}
-    ,
-    {G_TYPE_BOOLEAN, "capslock-toggle-chinese", PAGE_EDITING,
-     N_("Caps Lock toggles Chinese Mode"),
-     IBUS_CHEWING_PROPERTIES_SUBSECTION, "1", NULL, NULL, 0, 1,
-     capslockToggleChinese_apply_callback, 0,
-     N_("On: Caps Lock toggle Chinese/English\n"
-        "Off: Caps Lock only affect English letter case"), NULL}
+     NULL,
+     NULL
+     }
     ,
     {
      G_TYPE_STRING, "default-english-case", PAGE_EDITING,
      N_("Default English letter case\n"
-        "  (Only effective when Caps Lock toggles Chinese is ON)"),
+        "(Only effective when Caps Lock is the toggle key)"),
      IBUS_CHEWING_PROPERTIES_SUBSECTION, "lowercase",
      propDefaultEnglishLettercase_array, NULL, 0, 1,
      defaultEnglishLetterCase_apply_callback,
@@ -173,6 +171,20 @@ MkdgPropertySpec propSpecs[] = {
       "uppercase: Default to uppercase, press shift for lowercase."),
      NULL}
     ,
+    /* Sync between CapsLock and IM */
+    {G_TYPE_STRING, "sync-caps-lock", PAGE_EDITING,
+     N_("Sync between CapsLock and IM"),
+     IBUS_CHEWING_PROPERTIES_SUBSECTION, "keyboard", syncCapsLock_strs,
+     "Sync",
+     0, 1,
+     syncCapsLock_apply_callback,
+     MKDG_PROPERTY_FLAG_NO_NEW | MKDG_PROPERTY_FLAG_HAS_TRANSLATION,
+     N_
+     ("Occasionally, the CapsLock status does not match the IM, this option determines how these status be synchronized. Valid values:\n"
+      "\"disable\": Do nothing\n"
+      "\"keyboard\": IM status follows keyboard status\n"
+      "\"IM\": Keyboard status follows IM status"), NULL}
+    ,
     {
      G_TYPE_BOOLEAN, "plain-zhuyin", PAGE_SELECTING,
      N_("Plain Zhuyin mode"),
@@ -185,17 +197,25 @@ MkdgPropertySpec propSpecs[] = {
     {
      G_TYPE_UINT, "cand-per-page", PAGE_SELECTING,
      N_("Candidate per page"),
-     IBUS_CHEWING_PROPERTIES_SUBSECTION, "10", NULL, NULL, 8, 10,
+     IBUS_CHEWING_PROPERTIES_SUBSECTION, "5", NULL, NULL, 4, 10,
      candPerPage_apply_callback, 0,
      N_("Number of candidate per page."),
      NULL}
     ,
+    {
+     G_TYPE_BOOLEAN, "show-page-number", PAGE_SELECTING,
+     N_("Show page number"),
+     IBUS_CHEWING_PROPERTIES_SUBSECTION, "0", NULL, NULL, 0, 1,
+     showPageNumber_apply_callback, 0,
+     N_("Display the page number of the candidate list."),
+     NULL}
+    ,
     {
      G_TYPE_BOOLEAN, "phrase-choice-from-last", PAGE_SELECTING,
      N_("Choose phrases from backward"),
      IBUS_CHEWING_PROPERTIES_SUBSECTION, "1", NULL, NULL, 0, 1,
      phraseChoiceRearward_apply_callback, 0,
-     N_("Choose phrases from the back, without moving cursor."),
+     N_("Open candidate list from the back of a phrase, without moving the cursor to the front."),
      NULL}
     ,
     {
@@ -207,6 +227,15 @@ MkdgPropertySpec propSpecs[] = {
      "Press Space to select the candidate.",
      NULL}
     ,
+    {
+     G_TYPE_BOOLEAN, "vertical-lookup-table", PAGE_SELECTING,
+     N_("Vertical Lookup Table"),
+     IBUS_CHEWING_PROPERTIES_SUBSECTION,
+     "0", NULL, NULL, 0, 1,
+     verticalLookupTable_apply_callback, 0,
+     "Use vertical lookup table.",
+     NULL}
+    ,
     {
      G_TYPE_INVALID, "", "", "",
      IBUS_CHEWING_PROPERTIES_SUBSECTION, "", NULL, NULL, 0, 0,
