@@ -175,8 +175,9 @@ void self_key_sym_fix_test()
 {
     ibus_chewing_pre_edit_set_chi_eng_mode(self, FALSE);
 
-    ibus_chewing_pre_edit_set_property_boolean(self,
-                                               "capslock-toggle-chinese", TRUE);
+    ibus_chewing_pre_edit_set_property_string(self, "chi-eng-mode-toggle",
+                                              "caps_lock");
+
     ibus_chewing_pre_edit_set_property_string(self, "default-english-case",
                                               "no control");
     g_assert(self_key_sym_fix(self, '1', 0) == '1');
@@ -220,9 +221,9 @@ void self_key_sym_fix_test()
     g_assert(self_key_sym_fix(self, 'C', IBUS_SHIFT_MASK | IBUS_LOCK_MASK)
              == 'c');
 
-    ibus_chewing_pre_edit_set_property_boolean(self,
-                                               "capslock-toggle-chinese",
-                                               FALSE);
+    ibus_chewing_pre_edit_set_property_string(self, "chi-eng-mode-toggle",
+                                              "shift");
+
     ibus_chewing_pre_edit_set_property_string(self, "default-english-case",
                                               "no control");
     g_assert(self_key_sym_fix(self, 'd', 0) == 'd');
@@ -272,8 +273,10 @@ void self_key_sym_fix_test()
      * Chinese properly.
      */
     ibus_chewing_pre_edit_set_chi_eng_mode(self, TRUE);
-    ibus_chewing_pre_edit_set_property_boolean(self,
-                                               "capslock-toggle-chinese", TRUE);
+
+    ibus_chewing_pre_edit_set_property_string(self, "chi-eng-mode-toggle",
+                                              "caps_lock");
+
     ibus_chewing_pre_edit_set_property_string(self, "default-english-case",
                                               "no control");
     g_assert(self_key_sym_fix(self, 'e', 0) == 'e');
@@ -315,9 +318,9 @@ void self_key_sym_fix_test()
     g_assert(self_key_sym_fix(self, 'G', IBUS_SHIFT_MASK | IBUS_LOCK_MASK)
              == 'G');
 
-    ibus_chewing_pre_edit_set_property_boolean(self,
-                                               "capslock-toggle-chinese",
-                                               FALSE);
+    ibus_chewing_pre_edit_set_property_string(self, "chi-eng-mode-toggle",
+                                              "shift");
+
     ibus_chewing_pre_edit_set_property_string(self, "default-english-case",
                                               "no control");
     g_assert(self_key_sym_fix(self, 'h', 0) == 'h');
@@ -372,6 +375,7 @@ void self_handle_key_sym_default_test()
     assert_outgoing_pre_edit("", "ㄆ");
     ibus_chewing_pre_edit_clear(self);
 
+#if 0
     /*=== Test default-english-case handling ===*/
     ibus_chewing_pre_edit_set_chi_eng_mode(self, FALSE);
     ibus_chewing_pre_edit_set_apply_property_string(self,
@@ -400,6 +404,8 @@ void self_handle_key_sym_default_test()
     assert_outgoing_pre_edit("aaAAaaAA", "");
 
     ibus_chewing_pre_edit_clear(self);
+#endif
+
     assert_outgoing_pre_edit("", "");
 }
 
@@ -451,10 +457,14 @@ void process_key_text_with_symbol_test()
 }
 
 /* Mix english and chinese */
-/* " 這是ibus-chewing 輸入法"*/
+/* "這是ibus-chewing 輸入法"*/
 void process_key_mix_test()
 {
     TEST_CASE_INIT();
+
+    ibus_chewing_pre_edit_set_property_string(self, "chi-eng-mode-toggle",
+                                              "shift");
+
     key_press_from_string("5k4g4");
     key_press_from_key_sym(IBUS_KEY_Shift_L, 0);
     key_press_from_string("ibus-chewing ");
@@ -520,8 +530,11 @@ void process_key_down_arrow_test()
 /* Test shift then caps then caps then shift */
 /* String: 我要去 Brisbane 了。Daddy 好嗎 */
 /* Bug before 1.5.0 */
+/* Should be okay to remove this since we force users to choose between caps and shift */
 void process_key_shift_and_caps_test()
 {
+#if 0
+
     TEST_CASE_INIT();
     ibus_chewing_pre_edit_set_apply_property_boolean(self,
                                                      "plain-zhuyin", FALSE);
@@ -534,27 +547,26 @@ void process_key_shift_and_caps_test()
     ibus_chewing_pre_edit_set_apply_property_boolean(self,
                                                      "capslock-toggle-chinese",
                                                      TRUE);
+    ibus_chewing_pre_edit_set_apply_property_int(self,
+                                                 "max-chi-symbol-len", 33);
     g_assert(ibus_chewing_pre_edit_get_chi_eng_mode(self));
 
     key_press_from_string("ji3ul4fm4 ");
-    key_press_from_key_sym(IBUS_KEY_Return, 0);
-    assert_outgoing_pre_edit("我要去 ", "");
+    assert_outgoing_pre_edit("", "我要去 ");
 
     /* Shift: change to Eng-Mode */
     key_press_from_key_sym(IBUS_KEY_Shift_L, 0);
     g_assert(!ibus_chewing_pre_edit_get_chi_eng_mode(self));
     key_press_from_key_sym(IBUS_KEY_B, IBUS_SHIFT_MASK);        /* B */
     key_press_from_string("risbane ");
-    key_press_from_key_sym(IBUS_KEY_Return, 0);
-    assert_outgoing_pre_edit("我要去 Brisbane ", "");
+    assert_outgoing_pre_edit("", "我要去 Brisbane ");
 
     /* Caps Lock ON: change to Chi-Mode */
     key_press_from_key_sym(IBUS_KEY_Caps_Lock, 0);
     g_assert(ibus_chewing_pre_edit_get_chi_eng_mode(self));
     key_press_from_string("xk7");
     key_press_from_key_sym(IBUS_KEY_greater, IBUS_SHIFT_MASK);
-    key_press_from_key_sym(IBUS_KEY_Return, 0);
-    assert_outgoing_pre_edit("我要去 Brisbane 了。", "");
+    assert_outgoing_pre_edit("", "我要去 Brisbane 了。");
 
     /* Caps Lock OFF: change to Eng-Mode */
     key_press_from_key_sym(IBUS_KEY_Caps_Lock, 0);
@@ -566,11 +578,11 @@ void process_key_shift_and_caps_test()
     key_press_from_key_sym(IBUS_KEY_Shift_L, 0);
     g_assert(ibus_chewing_pre_edit_get_chi_eng_mode(self));
     key_press_from_string("cl3a87");
-    key_press_from_key_sym(IBUS_KEY_Return, 0);
-    assert_outgoing_pre_edit("我要去 Brisbane 了。Daddy 好嗎", "");
+    assert_outgoing_pre_edit("", "我要去 Brisbane 了。Daddy 好嗎");
 
     ibus_chewing_pre_edit_clear(self);
     assert_outgoing_pre_edit("", "");
+#endif
 }
 
 void full_half_shape_test()
@@ -625,9 +637,9 @@ void plain_zhuyin_shift_symbol_test()
     ibus_chewing_pre_edit_set_apply_property_boolean(self,
                                                      "plain-zhuyin", TRUE);
     g_assert(ibus_chewing_pre_edit_get_property_boolean(self, "plain-zhuyin"));
-    ibus_chewing_pre_edit_set_apply_property_boolean(self,
-                                                     "shift-toggle-chinese",
-                                                     TRUE);
+
+    ibus_chewing_pre_edit_set_property_string(self, "chi-eng-mode-toggle",
+                                              "shift");
 
     key_press_from_string("su31cl31");
 
@@ -658,11 +670,11 @@ void plain_zhuyin_shift_symbol_test()
     /* String is bypass in English mode */
     key_press_from_string("4321-9876 ");
 
-    assert_outgoing_pre_edit("你好，打電話；4321-9876 ", "");
+    assert_outgoing_pre_edit("你好，打電話；", "");
     key_press_from_key_sym(IBUS_KEY_Shift_L, IBUS_SHIFT_MASK);
     /* "來訂餐" */
     key_press_from_string("x9612u/42h0 2");
-    assert_outgoing_pre_edit("你好，打電話；4321-9876 來訂餐", "");
+    assert_outgoing_pre_edit("你好，打電話；來訂餐", "");
 
     ibus_chewing_pre_edit_clear(self);
     assert_outgoing_pre_edit("", "");
@@ -738,6 +750,9 @@ void test_arrow_keys_buffer_empty()
 /* GitHub #50: Cannot use Up, Down, PgUp, Ese ... etc. within "`" menu */
 
     TEST_CASE_INIT();
+    ibus_chewing_pre_edit_set_apply_property_boolean(self,
+                                                     "vertical-lookup-table",
+                                                     TRUE);
 
     key_press_from_string("`");
     g_assert(ibus_chewing_pre_edit_has_flag(self, FLAG_TABLE_SHOW));
@@ -750,18 +765,13 @@ void test_arrow_keys_buffer_empty()
     g_assert(chewing_cand_CurrentPage(self->context) == 1);
     key_press_from_key_sym(IBUS_KEY_Page_Up, 0);
     g_assert(chewing_cand_CurrentPage(self->context) == 0);
-    key_press_from_key_sym(IBUS_KEY_Up, 0);
+    key_press_from_key_sym(IBUS_KEY_Escape, 0);
     g_assert(!ibus_chewing_pre_edit_has_flag(self, FLAG_TABLE_SHOW));
 
     key_press_from_string("`");
     g_assert(ibus_chewing_pre_edit_has_flag(self, FLAG_TABLE_SHOW));
     key_press_from_key_sym(IBUS_KEY_BackSpace, 0);
     g_assert(!ibus_chewing_pre_edit_has_flag(self, FLAG_TABLE_SHOW));
-
-    key_press_from_string("`");
-    g_assert(ibus_chewing_pre_edit_has_flag(self, FLAG_TABLE_SHOW));
-    key_press_from_key_sym(IBUS_KEY_Escape, 0);
-    g_assert(!ibus_chewing_pre_edit_has_flag(self, FLAG_TABLE_SHOW));
 }
 
 void test_ctrl_1_open_candidate_list()
@@ -772,6 +782,7 @@ void test_ctrl_1_open_candidate_list()
 
     key_press_from_key_sym(IBUS_KEY_1, IBUS_CONTROL_MASK);
     g_assert(ibus_chewing_pre_edit_has_flag(self, FLAG_TABLE_SHOW));
+    key_press_from_key_sym(IBUS_KEY_Escape, 0);
 }
 
 void test_kp_eng_mode()
@@ -786,7 +797,7 @@ void test_kp_eng_mode()
     key_press_from_key_sym(IBUS_KP_1, 0);
     key_press_from_key_sym(IBUS_KEY_KP_9, 0);
     key_press_from_key_sym(IBUS_KP_0, 0);
-    assert_outgoing_pre_edit("190", "");
+    assert_outgoing_pre_edit("", "");
 }
 
 void test_kp_eng_mode_buffer()
@@ -815,7 +826,7 @@ void test_kp_chi_mode()
     key_press_from_key_sym(IBUS_KP_1, 0);
     key_press_from_key_sym(IBUS_KEY_KP_9, 0);
     key_press_from_key_sym(IBUS_KP_0, 0);
-    assert_outgoing_pre_edit("190", "");
+    assert_outgoing_pre_edit("", "");
 
     /* should remain chi-mode */
     g_assert(chewing_get_ChiEngMode(self->context) == 1);
